<?php
namespace Teknisa\PLA\Modules\Mymenu\Controllers;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Method;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO\Response\Error;

class Mym00106ParametrizacaoQrCodeController {
    protected $connection;
    protected $dataSourceName;
    
	public function __construct($connection) {
	    $this->connection = $connection;
	    $this->dataSourceName = "appconfig";
	}
	
	private function existsParams($NRORG) {
		$query = "SELECT NRAPPCONFIG "
				."FROM APPCONFIG "
				."WHERE NRORG = :P_NRORG";
		$queryParams = [
			"P_NRORG" => $NRORG
		];
		
		$statement = $this->connection->prepare($query);
		$statement->execute($queryParams);
		$resultSet = $statement->fetchAll();
		
		return ($resultSet)? true : false;
	}
	
	private function getNextId($primaryKey, $table) {
		$nextId = null;
		
		$query = "SELECT MAX(".$primaryKey.") AS ".$primaryKey." "
 	       		."FROM ".$table;
    	    
	    $statement = $this->connection->prepare($query);
	    $statement->execute([]);
	    $resultSet = $statement->fetchAll();
	    
	    
	    if ($resultSet)
	    	$nextId = intval($resultSet[0][$primaryKey]) + 1;
	    	
	    return $nextId;
	}
	
	private function getDNSInfo($NRORG) {
	    $query = "SELECT NRAPPCONFIG, "
	    		."		 DSDNS, "
	    		."		 DSPORTA "
	            ."FROM APPCONFIG "
	            ."WHERE NRORG = :P_NRORG";
        $queryParams = [
            "P_NRORG" => $NRORG
        ];
        
        $statement = $this->connection->prepare($query);
        $statement->execute($queryParams);
        $resultSet = $statement->fetchAll();
        
        return $resultSet[0];
	}
	
	private function getQrCodeDescription($NRQRCODELOCALPESQ, $NRORG) {
		$query = "SELECT DSQRCODEAPP "
				."FROM QRCODELOCALPESQ "
				."WHERE NRQRCODELOCALPESQ = :P_NRQRCODELOCALPESQ "
				."	AND NRORG = :P_NRORG";
		$queryParams = [
			"P_NRQRCODELOCALPESQ"	=> $NRQRCODELOCALPESQ,
			"P_NRORG"				=> $NRORG
		];
		
		$statement = $this->connection->prepare($query);
		$statement->execute($queryParams);
		$resultSet = $statement->fetchAll();
		
		return ( isset($resultSet[0]["DSQRCODEAPP"]) )? $resultSet[0]["DSQRCODEAPP"] : NULL;
	}
	
	private function setQrCodeDescription($NRQRCODELOCALPESQ, $DSQRCODEAPP, $NRORG) {
		$query = "UPDATE QRCODELOCALPESQ "
				."SET DSQRCODEAPP = :P_DSQRCODEAPP "
				."WHERE NRQRCODELOCALPESQ = :P_NRQRCODELOCALPESQ "
				."	AND NRORG = :P_NRORG";
		$queryParams = [
			"P_NRQRCODELOCALPESQ"	=> $NRQRCODELOCALPESQ,
			"P_DSQRCODEAPP"			=> $DSQRCODEAPP,
			"P_NRORG"				=> $NRORG
		];
		
		$statement = $this->connection->prepare($query);
		$statement->execute($queryParams);
	}
	
	public function regenerateQrCodes(Request $request, Response $response) {
		
		try {
			$row = $request->getRow();
			
			$query = "UPDATE QRCODELOCALPESQ "
					."SET DSQRCODEAPP = NULL "
					."WHERE INSTR( :P_NRQRCODELOCALPESQ, NRQRCODELOCALPESQ) <> 0";
			$queryParams = [
				"P_NRQRCODELOCALPESQ" => implode("_",$row["NRQRCODELOCALPESQ"])
			];
			
			$statement = $this->connection->prepare($query);
			$statement->execute($queryParams);
			
			$response->addDataSet(new DataSet("data",["QrCodes regerados com sucesso"]));
		} catch(\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	    }
		
	}
	
	public function generateQrCode(Request $request, Response $response) {
		
		try {
		$row = $request->getRow();
		
		if ($row["regenerate"])
			$qrCodeDescription = null;
		else
			$qrCodeDescription = $this->getQrCodeDescription($row["NRQRCODELOCALPESQ"], $row["NRORG"]);
		
		if (!$qrCodeDescription) {
			$appconfig = $this->getDNSInfo($row["NRORG"]);
			$qrCodeDescription = $appconfig["DSDNS"].";".$appconfig["DSPORTA"].";".$row["NRQRCODELOCALPESQ"].";".$row["NRORG"];
			
			$this->setQrCodeDescription($row["NRQRCODELOCALPESQ"], $qrCodeDescription, $row["NRORG"]);
		}

		$response->addDataSet(new DataSet("data",[$qrCodeDescription]));
		} catch(\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
		}
	}
	
	public function getAppConfig(Request $request, Response $response) {
		$row = $request->getRow();
	    
	    $resultSet = $this->getDNSInfo($row["NRORG"]);
	    
        $response->addDataSet(new DataSet("data", $resultSet));
	}
	
	public function saveParams(Request $request, Response $response ) {
	    
	    try {
    	    $row = $request->getRow();
			
    	    if ($this->existsParams($row["NRORG"])) {
    	        $query = "UPDATE APPCONFIG "
    	        		."SET DSDNS = :P_DSDNS, "
    	        		."	  DSPORTA = :P_DSPORTA, "
    	        		."	  DTULTATU = :P_DTULTATU "
    	        		."WHERE NRORG = :P_NRORG";
    	        		
    	        $queryParams = [
	                "P_DSDNS" 		=> $row["DSDNS"],
	                "P_DSPORTA"		=> $row["DSPORTA"],
	                "P_DTULTATU"	=> date("d-m-Y H:m:s"),
	                "P_NRORG" 		=> $row["NRORG"]
	            ];
    	    }
    	    else {
        	    $row["NRAPPCONFIG"] = $this->getNextId("NRAPPCONFIG", "APPCONFIG");
        	    
        	    $query = "INSERT INTO APPCONFIG(NRAPPCONFIG, DSDNS, DSPORTA, IDAPLICATIVO, IDATIVO, DTINCLUSAO, DTULTATU, NRORG, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU) "
        	    		."VALUES (:P_NRAPPCONFIG, :P_DSDNS, :P_DSPORTA, :P_IDAPLICATIVO, :P_IDATIVO, :P_DTINCLUSAO, :P_DTULTATU, :P_NRORG, :P_NRORGINCLUSAO, :P_CDOPERINCLUSAO, :P_NRORGULTATU, :P_CDOPERULTATU)";
        	    
				$queryParams = [
					"P_NRAPPCONFIG" 	=> $row["NRAPPCONFIG"],
					"P_DSDNS"			=> $row["DSDNS"],
					"P_DSPORTA"			=> $row["DSPORTA"],
					"P_IDAPLICATIVO"	=> "MYMENU",
					"P_IDATIVO"			=> "S",
					"P_DTINCLUSAO"		=> date("d-m-Y H:m:s"),
					"P_DTULTATU"		=> date("d-m-Y H:m:s"),
					"P_NRORG"			=> $row["NRORG"],
					"P_NRORGINCLUSAO" 	=> 1.0,
					"P_CDOPERINCLUSAO"	=> "1",
					"P_NRORGULTATU"		=> 1,
					"P_CDOPERULTATU"	=> 000000000999
				];
    	    }
    	    
    	    $statement = $this->connection->prepare($query);
    	    $statement->execute($queryParams);
    	    
    	    $response->addDataSet(new DataSet("data",["Salvo com sucesso"]));
	    } catch(\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	    }
	}

	public function saveForm(Request $request, Response $response) {
		try {
			$row = $request->getRow();
			
			if ($row["NRQRCODELOCALPESQ"]) {
				$query = "UPDATE QRCODELOCALPESQ "
						."SET NRLOCALPESQ = :P_NRLOCALPESQ, "
						."	  DSQRCODEAPP = :P_DSQRCODEAPP, " 
						."	  QTDIASVALIDADE = :P_QTDIASVALIDADE, "
						."	  DTEXPIRACAO = :P_DTEXPIRACAO, "
						."	  DTULTATU = :P_DTULTATU "
						."WHERE NRQRCODELOCALPESQ = :P_NRQRCODELOCALPESQ"
						."	AND NRORG = :P_NRORG";
				
				$queryParams = [
					"P_NRQRCODELOCALPESQ"	=> $row["NRQRCODELOCALPESQ"],
					"P_NRORG"				=> $row["NRORG"],
					"P_NRLOCALPESQ" 		=> $row["NRLOCALPESQ"],
					"P_DSQRCODEAPP" 		=> $row["DSQRCODEAPP"],
					"P_QTDIASVALIDADE"		=> $row["QTDIASVALIDADE"],
					"P_DTEXPIRACAO" 		=> $row["DTEXPIRACAO"],	
					"P_DTULTATU"			=> date("d-m-Y H:m:s")
				];
			}
			else {
				$row["NRQRCODELOCALPESQ"] = $this->getNextId("NRQRCODELOCALPESQ", "QRCODELOCALPESQ");
				
				$query = "INSERT INTO QRCODELOCALPESQ(NRQRCODELOCALPESQ, CDFILIAL, NRLOCALPESQ, DSQRCODEAPP, QTDIASVALIDADE, DTEXPIRACAO, IDATIVO, DTINCLUSAO, DTULTATU, NRORG, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU) "
						."VALUES (:P_NRQRCODELOCALPESQ, :P_CDFILIAL, :P_NRLOCALPESQ, :P_DSQRCODEAPP, :P_QTDIASVALIDADE, :P_DTEXPIRACAO, :P_IDATIVO, :P_DTINCLUSAO, :P_DTULTATU, :P_NRORG, :P_NRORGINCLUSAO, :P_CDOPERINCLUSAO, :P_NRORGULTATU, :P_CDOPERULTATU)";
				
				$queryParams = [
					"P_NRQRCODELOCALPESQ"	=> $row["NRQRCODELOCALPESQ"],
					"P_CDFILIAL"			=> $row["CDFILIAL"],
					"P_NRLOCALPESQ" 		=> $row["NRLOCALPESQ"],
					"P_DSQRCODEAPP" 		=> $row["DSQRCODEAPP"],
					"P_QTDIASVALIDADE"		=> $row["QTDIASVALIDADE"],
					"P_DTEXPIRACAO" 		=> $row["DTEXPIRACAO"],
					"P_IDATIVO" 			=> "S",
					"P_DTINCLUSAO"			=> date("d-m-Y H:m:s"),
					"P_DTULTATU"			=> date("d-m-Y H:m:s"),
					"P_NRORG"				=> $row["NRORG"],
					"P_NRORGINCLUSAO"		=> 1.0,
					"P_CDOPERINCLUSAO"		=> "1",
					"P_NRORGULTATU" 		=> 1,
					"P_CDOPERULTATU"		=> 1
				];
			}
				
			$statement = $this->connection->prepare($query);
			$statement->execute($queryParams);
			
		    $response->addDataSet(new DataSet("data",["Salvo com sucesso", $row["NRQRCODELOCALPESQ"]]));
		} catch(\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	    }
	}
	
	public function delete(Request $request, Response $response) {
		$row = $request->getRow();
		
		$query = "DELETE FROM QRCODELOCALPESQ "
				."WHERE NRQRCODELOCALPESQ = :P_NRQRCODELOCALPESQ "
				."	AND NRORG = :P_NRORG";
		$queryParams = [
			"P_NRQRCODELOCALPESQ" => $row["NRQRCODELOCALPESQ"],
			"P_NRORG"			  => $row["NRORG"]
		];
		
		$statement = $this->connection->prepare($query);
		$statement->execute($queryParams);
		
		$response->addDataSet(new DataSet("data", ["Excluído com sucesso"]));
	}
}