<?php
namespace Teknisa\PLA\Modules\Requisicao\Services;
use Teknisa\GER\Utilities\Environment;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Teknisa\PLA\Modules\Requisicao\Services\Pla01800RequisicaoGeralRepository;
use Teknisa\PLA\Modules\Requisicao\Services\Pla01800RequisicaoGeralRemoteRepository;

class Pla01800RequisicaoGeralService{

    protected $novoCodigo;
    protected $environment;
    protected $pla01800repository;
    protected $pla01800remoteRepository;

    public function __construct($novoCodigo, Environment $environment,
                                Pla01800RequisicaoGeralRepository $pla01800repository,
                                Pla01800RequisicaoGeralRemoteRepository $pla01800remoteRepository){
        $this->novoCodigo = $novoCodigo;
        $this->environment = $environment;
        $this->pla01800repository = $pla01800repository;
        $this->pla01800remoteRepository = $pla01800remoteRepository;
    }

    public function montaListaFiliServ($nrorg, $filiaisServ){
        $nrlistaFiliServ = $this->novoCodigo->getCodeByTableName("LISTA", $nrorg);
        foreach($filiaisServ as $filialServ){
            $filialServicoParametros = array(
                "NRORG" => $nrorg, "NRLISTA" => $nrlistaFiliServ, "CDLISTA" => $filialServ
                , "DTULTATULIST" => '', "NRORGINCLUSAO" => '', "DTINCLUSAO" => ''
                , "CDOPERINCLUSAO" => '', "IDATIVO" => '', "CDOPERULTATU" => ''
                , "DTULTATU" => '', "NRORGULTATU" => '', "DTULTATUPRFIL" => ''
                , "__is_new" => true
            );
            $this->pla01800repository->saveLista($filialServicoParametros);
        }
        return $nrlistaFiliServ;
    }
    
    public function clearLista($row){
        $this->pla01800repository->deleteLista($row["NRLISTA"], $row["NRORG"]);
    }

    public function clearConsplan($nrconsplan){
        $this->pla01800repository->deleteConsplan($nrconsplan);
    }

    public function calculaRequisicaoGeral(&$params){
    
        $nrorg = $params["NRORG"];
        $cdfilial = $params["LISTAFILIAL"];
        $cdoperador = $params["CDOPERADOR"];
        $cdgrupfili = $params["CDGRUPFILI"];
        $nrconsplan = $params["NRCONSPLAN"];
        $servicoFilial = $params["SERVICOFILIAL"];
        
        $nrlistaFiliServ = $this->montaListaFiliServ($nrorg, $servicoFilial);
        $params["NRLISTA"] = $nrlistaFiliServ;
        $mensagemErro = $this->pla01800repository->verificaParametrizacao($nrlistaFiliServ);
        if(!empty($mensagemErro)){
            return $this->trataErros($mensagemErro);
        }
        if($params['CD_REPORT_NAME'] == 'PLA01806'){
            $result = $this->pla01800repository->verificaLista($nrlistaFiliServ);
            if(!empty($result)){
                return "Existe(m) serviço(s) que não utiliza(m) requisição/retirada por prato.";
            }
        }
        $params['LISTASERVFILI'] = $nrlistaFiliServ;
        $this->pla01800repository->deleteConsplanLista($nrconsplan, $nrlistaFiliServ);
        $params["NRCONSPLAN"] = $this->novoCodigo->getCodeByTableName("CONSPLAN".$this->environment->getCdFilial(), $nrorg);
        $this->pla01800repository->executaRequisicao($params);
        return null;
    }

    public function retornaServicos($row){
        
        $nrorg = $row["NRORG"];
        $servicoFilial = $row["SERVICOFILIAL"];
        if($row['T_CDSERVFILI'] === 'T'){
            $row['CDFILIAL'] = ($row['T_CDFILIAL'] === "T"? $row['T_CDFILIAL'] : $row['LISTAFILIAL']);
            $row['CDGRUPSERV'] = ($row['T_CDGRUPSERV'] === "T"? $row['T_CDGRUPSERV'] : $row['LISTAGRUPSERV']);
            
            //valores que o filtro de serviços precisa
            $fieldsToFilter = array("NRORG", "CDFILIAL", "CDOPERADOR", "CDGRUPSERV", "IDTIPOSERV");
            $paramObjectArray = $this->buildFilterArrayFromRow((array)$row, $fieldsToFilter);

            $row['SERVICOFILIAL'] = $this->pla01800remoteRepository->fetchServicos($paramObjectArray);
        }
        
        return $this->concatArray($row['SERVICOFILIAL'], 'CDSERVFILI');
    }

    public function retornaFiliais($row){
        if($row['T_CDFILIAL'] === 'T'){
            $row['CDGRUPFILI'] = ($row['CDGRUPFILI'] === ""? "T" : $row['CDGRUPFILI']);

            //valores que o filtro de serviços precisa
            $fieldsToFilter = array("NRORG", "CDOPERADOR", "CDGRUPFILI");

            $paramObjectArray = $this->buildFilterArrayFromRow((array)$row, $fieldsToFilter);
            $row['CDFILIAL'] = $this->pla01800remoteRepository->fetchFiliais($paramObjectArray);
        }
        return $this->concatArray($row['CDFILIAL']);
    }

    public function retornaGrupoServicos($row){
        if($row['T_CDGRUPSERV'] === 'T'){
            $row['CDGRUPSERV'] = $this->pla01800remoteRepository->fetchGrupoServicos();
        }
        return $this->concatArray($row['CDGRUPSERV']);
    }

    public function retornaTipoPrato($row){
        if($row['TCDTIPOPRAT'] === 'T'){
            $row['CDTIPOPRAT'] = $this->pla01800remoteRepository->fetchTipoPrato();
        }
        return $this->concatArray($row['CDTIPOPRAT']);
    }

    /*
     * Constroi um array de arrays a partir de uma row, no formato chave=>valor,
     * e um array de strings especificando quais das colunas deverão ser usadas
     */
    public function buildFilterArrayFromRow($row, $filterKeys){

        //pega todos os valores de $row com chaves presentes em $filterKeys
        //o array_flip faz com que os valores de $filterKeys sejam chaves
        $paramValueArray = array_intersect_key($row, array_flip($filterKeys));

        return array_map(function($key, $value){
            return array("columnName"=>$key, "operator"=>"=", "value"=>$value);
        }, array_keys($paramValueArray), $paramValueArray);
    }

    public function concatArray($array, $key = false){
        if ($key){
            $string = '';
            foreach ($array as $item) {
              $string .= $item[$key].'_';
            }
            return trim($string, '_');
        }
        else return implode($array, "_");
    }


    public function trataErros($mensagemErro){
        if($mensagemErro == "TP_CONSFIXO"){
            return "Tipo de prato default de consumo fixo deve ser parametrizado.";
        }
        if($mensagemErro == "PRATO_CONSFIXO"){
            return "Prato default de consumo fixo deve ser parametrizado.";
        }
        if($mensagemErro == "TP_CONSPER"){
            return "Tipo de prato default de consumo percapita deve ser parametrizado.";
        }
        if($mensagemErro == "PRATO_CONSPER"){
            return "Prato default de consumo percapita deve ser parametrizado.";
        }
    }
    
    public function getReportProductMask($toReplace = '@'){
      $mask = $this->pla01800repository->getProductMask();
      return '!'.preg_replace('/[^\.]/i', $toReplace, $mask);
    }
}